use iced::widget::{button, center, checkbox, column, row, scrollable, text_input, Space, Text};
use iced::window::Settings;
use iced::{Element, Length, Size, Theme};
use std::collections::HashMap;

struct TaskData {
    checked: bool,
    value: String,
}

fn main() -> iced::Result {
    let settings = Settings {
        size: Size::new(500.0, 600.0),
        resizable: false,
        ..Settings::default()
    };
    
    iced::application("To Do App", Todo::update, Todo::view)
        .theme(Todo::theme)
        .window(settings)
        .run()
}

#[derive(Default)]
struct Todo {
    new_task: String,
    tasks: HashMap<usize, TaskData>,
}

#[derive(Debug, Clone)]
enum Message {
    AddTask,
    DeleteTask(usize),
    ActivityTask,
    ContentUpdated(String),
}

impl Todo {
    fn update(&mut self, message: Message) {
        match message {
            Message::ContentUpdated(new_value) => self.new_task = new_value,
            Message::AddTask => {
                if self.new_task.is_empty() { return; }
                
                let data = TaskData {
                    checked: false,
                    value: self.new_task.to_string(),
                };

                self.tasks.insert(self.tasks.len(), data);


                // temp disable for testing fixme
                //self.new_task = String::from("");
            },
            Message::DeleteTask(id) => todo!(),
            Message::ActivityTask => todo!(),
        }
    }
    
    fn view(&self) -> Element<Message> {
        
        let input = text_input("Enter new task", &self.new_task)
            .width(300)
            .size(25)
            .on_input(Message::ContentUpdated);

        
        let add_btn = button(
            Text::new("Add task")
                .size(19)
                .center()
            )
            .width(Length::Shrink)
            .padding(10)
            .on_press(Message::AddTask);
        
        let new_task = row![input, add_btn].spacing(10);
        let mut saved_tasks = column![];

        for task in &self.tasks {
            let chk = checkbox("", task.1.checked).size(25);
            let label = Text::new(&task.1.value).width(200);
            let edit = button("edit").width(Length::Shrink);
            let delete = button("delete").width(Length::Shrink);
            let task_line = row![chk, label, edit, delete].spacing(10);
            
            saved_tasks = saved_tasks.push(task_line.padding(10));
        }

        let r = center(column![new_task, scrollable(saved_tasks).spacing(10), Space::with_height(Length::Fill)].spacing(20));
        r.into()
    }
    
    fn theme(&self) -> Theme {
        Theme::Dark
    }
}